"""Argparser module for hub login"""
import os

from ..helper import add_arg_group


def mixin_hub_docker_login_parser(parser):
    """Add the options for the docker login

    :param parser: the parser
    """
    gp = add_arg_group(parser, title='Docker login')

    gp.add_argument(
        '--username',
        type=str,
        help='The Docker registry username',
        default=os.environ.get('JINAHUB_USERNAME', ''),
    )
    # _gp = parser.add_mutually_exclusive_group()
    # _gp.add_argument('--password-stdin', type=argparse.FileType('r'),
    #                  default=(sys.stdin if sys.stdin.isatty() else None),
    #                  help='take the password from stdin')
    gp.add_argument(
        '--password',
        type=str,
        help='The plaintext password',
        default=os.environ.get('JINAHUB_PASSWORD', ''),
    )
    gp.add_argument(
        '--registry',
        type=str,
        default='https://index.docker.io/v1/',
        help='The URL to the Docker registry, e.g. https://index.docker.io/v1/',
    )
    gp.add_argument(
        '--repository',
        type=str,
        default='jinahub',
        help='The Docker repository name, change this when pushing image to a personal repository',
    )



"""Module wrapping the Client of Jina."""
from .base import BaseClient, CallbackFnType, InputType, InputDeleteType
from .helper import callback_exec
from .mixin import PostMixin
from .request import GeneratorSourceType
from .websocket import WebSocketClientMixin

__all__ = ['Client', 'WebSocketClient']


class Client(PostMixin, BaseClient):
    """A simple Python client for connecting to the gRPC gateway.

    It manages the asyncio event loop internally, so all interfaces are synchronous from the outside.
    """

    @property
    def client(self) -> 'Client':
        """Return the client object itself

        .. # noqa: DAR201"""
        return self


class WebSocketClient(Client, WebSocketClientMixin):
    """A Python Client to stream requests from a Flow with a REST Gateway.

    :class:`WebSocketClient` shares the same interface as :class:`Client` and provides methods like
    :meth:`index`, "meth:`search`, :meth:`train`, :meth:`update` & :meth:`delete`.

    It is used by default while running operations when we create a `Flow` with `rest_api=True`

    .. highlight:: python
    .. code-block:: python

        from jina.flow import Flow
        f = Flow(rest_api=True).add().add()

        with f:
            f.index(['abc'])


    :class:`WebSocketClient` can also be used to run operations for a remote Flow

    .. highlight:: python
    .. code-block:: python

        # A Flow running on remote
        from jina.flow import Flow
        f = Flow(rest_api=True, port_expose=34567).add().add()

        with f:
            f.block()

        # Local WebSocketClient running index & search
        from jina.clients import WebSocketClient

        client = WebSocketClient(...)
        client.index(...)
        client.search(...)


    :class:`WebSocketClient` internally handles an event loop to run operations asynchronously.
    """
